use {
    crate::{Error, B16_ALPHABET},
    std::io::{ErrorKind, Read, Write},
};

pub struct Encoder<R: Read, W: Write> {
    reader: R,
    writer: W,
}

impl<R: Read, W: Write> Encoder<R, W> {
    /// Creates a new encoder with the given reader and writer. If alphabet is
    /// `None` the encoder will use the default rfc4648 alphabet.
    pub fn new(reader: R, writer: W) -> Self {
        Self { reader, writer }
    }

    /// Encodes the given data as hexadecimal
    /// # Errors
    /// May return an error on IO failure
    pub fn encode(&mut self) -> Result<(), Error> {
        loop {
            let mut ibuf = [0; 1];
            let mut obuf = [' '; 2];
            let mut n_bytes = 0;
            loop {
                n_bytes += match self.reader.read(&mut ibuf) {
                    Ok(n) => n,
                    Err(e) if e.kind() == ErrorKind::Interrupted => continue,
                    Err(e) => return Err(e.into()),
                };
                break;
            }
            if n_bytes == 0 {
                break;
            }
            let mut idx = usize::from(ibuf[0] & 0b1111);
            obuf[1] = B16_ALPHABET[idx];
            idx = usize::from((ibuf[0] & 0b1111_0000) >> 4);
            obuf[0] = B16_ALPHABET[idx];
            write!(self.writer, "{}{}", obuf[0], obuf[1])?;
        }
        Ok(())
    }

    pub fn output(self) -> W {
        self.writer
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn encode() {
        let mut encoder = Encoder::new("Hello, World".as_bytes(), vec![]);
        encoder.encode().unwrap();
        assert_eq!(encoder.output(), b"48656C6C6F2C20576F726C64");
        encoder = Encoder {
            reader: "Hello, World!".as_bytes(),
            writer: vec![],
        };
        encoder.encode().unwrap();
        assert_eq!(encoder.output(), b"48656C6C6F2C20576F726C6421");
        encoder = Encoder::new("Hello, World!\n".as_bytes(), vec![]);
        encoder.encode().unwrap();
        assert_eq!(encoder.output(), b"48656C6C6F2C20576F726C64210A");
    }
}
